package serialization;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.net.Socket;
import java.net.UnknownHostException;

import serialization.PrimeFindServer.GiveTaskResponse;

public class PrimeFindClient {

	public static abstract class ClientMessage implements Serializable {

		private static final long serialVersionUID = 1L;

		public static class ClientAskMessage extends ClientMessage {

			private static final long serialVersionUID = 1L;
		}

		public static class ClientPutMessage extends ClientMessage {
			
			private static final long serialVersionUID = 1L;
			
			private PrimeFinder state;

			public ClientPutMessage(PrimeFinder state) {
				this.state = state;
			}

			public PrimeFinder getState() {
				return state;
			}
		}
	}

	public static void main(String[] args) {
		PrimeFindClient client = new PrimeFindClient();
		client.run();
	}

	private void run() {
		try (Socket socket = new Socket("127.0.0.1", 4444);
				ObjectOutputStream out = new ObjectOutputStream(socket.getOutputStream());
				ObjectInputStream in = new ObjectInputStream(socket.getInputStream());) {
			out.writeObject(new ClientMessage.ClientAskMessage());
			Object resp = in.readObject();
			if (resp instanceof GiveTaskResponse) {
				Thread terminationCheck = new Thread(() -> {
					BufferedReader buf = new BufferedReader(new InputStreamReader(System.in));
					try {
						buf.readLine();
					} catch (Exception e) {
						e.printStackTrace();
					}
				});
				terminationCheck.start();
				PrimeFinder finder = ((GiveTaskResponse) resp).getFinder();
				while (terminationCheck.isAlive()) {
					finder.step();
				}
				out.writeObject(new ClientMessage.ClientPutMessage(finder));
			}
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

	}

}
